@@ -21,7 +21,7 @@ from typing import TYPE_CHECKING, Callable, TypeVar
 
 import _string
 import astroid.objects
-from astroid import TooManyLevelsError, nodes
+from astroid import TooManyLevelsError, nodes, util
 from astroid.context import InferenceContext
 from astroid.exceptions import AstroidError
 from astroid.nodes._base_nodes import ImportNode
@@ -885,7 +885,7 @@ def decorated_with(
             if any(
                 i.name in qnames or i.qname() in qnames
                 for i in decorator_node.infer()
-                if i is not None and i != astroid.Uninferable
+                if i is not None and not isinstance(i, util.UninferableBase)
             ):
                 return True
         except astroid.InferenceError:
@@ -936,11 +936,10 @@ def uninferable_final_decorators(
             decorator, "attrname", None
         ) == "final"
 
-        if (is_from_import or is_import) and safe_infer(decorator) in [
-            astroid.Uninferable,
-            None,
-        ]:
-            decorators.append(decorator)
+        if is_from_import or is_import:
+            inferred = safe_infer(decorator)
+            if inferred is None or isinstance(inferred, util.UninferableBase):
+                decorators.append(decorator)
     return decorators
 
 
@@ -1373,7 +1372,7 @@ def safe_infer(
     except Exception as e:  # pragma: no cover
         raise AstroidError from e
 
-    if value is not astroid.Uninferable:
+    if not isinstance(value, util.UninferableBase):
         inferred_types.add(_get_python_type_of_node(value))
 
     # pylint: disable = too-many-try-statements
@@ -1458,7 +1457,7 @@ def node_type(node: nodes.NodeNG) -> SuccessfulInferenceResult | None:
     types: set[SuccessfulInferenceResult] = set()
     try:
         for var_type in node.infer():
-            if var_type == astroid.Uninferable or is_none(var_type):
+            if isinstance(var_type, util.UninferableBase) or is_none(var_type):
                 continue
             types.add(var_type)
             if len(types) > 1:
@@ -1812,7 +1811,9 @@ def get_import_name(importnode: ImportNode, modname: str | None) -> str | None:
         root = importnode.root()
         if isinstance(root, nodes.Module):
             try:
-                return root.relative_to_absolute_name(modname, level=importnode.level)  # type: ignore[no-any-return]
+                return root.relative_to_absolute_name(  # type: ignore[no-any-return]
+                    modname, level=importnode.level
+                )
             except TooManyLevelsError:
                 return modname
     return modname
@@ -2056,7 +2057,7 @@ def is_hashable(node: nodes.NodeNG) -> bool:
     # pylint: disable = too-many-try-statements
     try:
         for inferred in node.infer():
-            if inferred is astroid.Uninferable or isinstance(inferred, nodes.ClassDef):
+            if isinstance(inferred, (nodes.ClassDef, util.UninferableBase)):
                 return True
             if not hasattr(inferred, "igetattr"):
                 return True
@@ -2193,6 +2194,53 @@ def is_class_attr(name: str, klass: nodes.ClassDef) -> bool:
         return False
 
 
+def is_defined(name: str, node: nodes.NodeNG) -> bool:
+    """Searches for a tree node that defines the given variable name."""
+    is_defined_so_far = False
+
+    if isinstance(node, nodes.NamedExpr):
+        is_defined_so_far = node.target.name == name
+
+    if isinstance(node, (nodes.Import, nodes.ImportFrom)):
+        is_defined_so_far = any(node_name[0] == name for node_name in node.names)
+
+    if isinstance(node, nodes.With):
+        is_defined_so_far = any(
+            isinstance(item[1], nodes.AssignName) and item[1].name == name
+            for item in node.items
+        )
+
+    if isinstance(node, (nodes.ClassDef, nodes.FunctionDef)):
+        is_defined_so_far = node.name == name
+
+    if isinstance(node, nodes.AnnAssign):
+        is_defined_so_far = (
+            node.value
+            and isinstance(node.target, nodes.AssignName)
+            and node.target.name == name
+        )
+
+    if isinstance(node, nodes.Assign):
+        is_defined_so_far = any(
+            any(
+                (
+                    (
+                        isinstance(elt, nodes.Starred)
+                        and isinstance(elt.value, nodes.AssignName)
+                        and elt.value.name == name
+                    )
+                    or (isinstance(elt, nodes.AssignName) and elt.name == name)
+                )
+                for elt in get_all_elements(target)
+            )
+            for target in node.targets
+        )
+
+    return is_defined_so_far or any(
+        is_defined(name, child) for child in node.get_children()
+    )
+
+
 def get_inverse_comparator(op: str) -> str:
     """Returns the inverse comparator given a comparator.
 
