@@ -10,7 +10,7 @@ import re
 
 import astroid
 from astroid import nodes
-from astroid.util import Uninferable
+from astroid.util import UninferableBase
 
 from pylint.checkers import utils
 
@@ -43,7 +43,7 @@ def get_setters_property_name(node: nodes.FunctionDef) -> str | None:
             and decorator.attrname == "setter"
             and isinstance(decorator.expr, nodes.Name)
         ):
-            return decorator.expr.name
+            return decorator.expr.name  # type: ignore[no-any-return]
     return None
 
 
@@ -89,7 +89,7 @@ def returns_something(return_node: nodes.Return) -> bool:
     return not (isinstance(returns, nodes.Const) and returns.value is None)
 
 
-def _get_raise_target(node: nodes.NodeNG) -> nodes.NodeNG | Uninferable | None:
+def _get_raise_target(node: nodes.NodeNG) -> nodes.NodeNG | UninferableBase | None:
     if isinstance(node.exc, nodes.Call):
         func = node.exc.func
         if isinstance(func, (nodes.Name, nodes.Attribute)):
@@ -126,7 +126,7 @@ def possible_exc_types(node: nodes.NodeNG) -> set[nodes.ClassDef]:
         if handler and handler.type:
             try:
                 for exception in astroid.unpack_infer(handler.type):
-                    if exception is not astroid.Uninferable:
+                    if not isinstance(exception, UninferableBase):
                         exceptions.append(exception)
             except astroid.InferenceError:
                 pass
