@@ -4,8 +4,12 @@
 
 """Utilities for creating VCG and Dot diagrams."""
 
+from __future__ import annotations
+
+import argparse
 import itertools
 import os
+from collections.abc import Iterable
 
 from astroid import modutils, nodes
 
@@ -16,7 +20,7 @@ from pylint.pyreverse.diagrams import (
     PackageDiagram,
     PackageEntity,
 )
-from pylint.pyreverse.printer import EdgeType, NodeProperties, NodeType
+from pylint.pyreverse.printer import EdgeType, NodeProperties, NodeType, Printer
 from pylint.pyreverse.printer_factory import get_printer_for_filetype
 from pylint.pyreverse.utils import is_exception
 
@@ -24,36 +28,17 @@ from pylint.pyreverse.utils import is_exception
 class DiagramWriter:
     """Base class for writing project diagrams."""
 
-    def __init__(self, config):
+    def __init__(self, config: argparse.Namespace) -> None:
         self.config = config
         self.printer_class = get_printer_for_filetype(self.config.output_format)
-        self.printer = None  # defined in set_printer
+        self.printer: Printer  # defined in set_printer
         self.file_name = ""  # defined in set_printer
         self.depth = self.config.max_color_depth
-        self.available_colors = itertools.cycle(
-            [
-                "aliceblue",
-                "antiquewhite",
-                "aquamarine",
-                "burlywood",
-                "cadetblue",
-                "chartreuse",
-                "chocolate",
-                "coral",
-                "cornflowerblue",
-                "cyan",
-                "darkgoldenrod",
-                "darkseagreen",
-                "dodgerblue",
-                "forestgreen",
-                "gold",
-                "hotpink",
-                "mediumspringgreen",
-            ]
-        )
-        self.used_colors = {}
+        # default colors are an adaption of the seaborn colorblind palette
+        self.available_colors = itertools.cycle(self.config.color_palette)
+        self.used_colors: dict[str, str] = {}
 
-    def write(self, diadefs):
+    def write(self, diadefs: Iterable[ClassDiagram | PackageDiagram]) -> None:
         """Write files for <project> according to <diadefs>."""
         for diagram in diadefs:
             basename = diagram.title.strip().replace(" ", "_")
@@ -61,10 +46,10 @@ class DiagramWriter:
             if os.path.exists(self.config.output_directory):
                 file_name = os.path.join(self.config.output_directory, file_name)
             self.set_printer(file_name, basename)
-            if diagram.TYPE == "class":
-                self.write_classes(diagram)
-            else:
+            if isinstance(diagram, PackageDiagram):
                 self.write_packages(diagram)
+            else:
+                self.write_classes(diagram)
             self.save()
 
     def write_packages(self, diagram: PackageDiagram) -> None:
@@ -88,7 +73,7 @@ class DiagramWriter:
     def write_classes(self, diagram: ClassDiagram) -> None:
         """Write a class diagram."""
         # sorted to get predictable (hence testable) results
-        for obj in sorted(diagram.objects, key=lambda x: x.title):
+        for obj in sorted(diagram.objects, key=lambda x: x.title):  # type: ignore[no-any-return]
             obj.fig_id = obj.node.qname()
             type_ = NodeType.INTERFACE if obj.shape == "interface" else NodeType.CLASS
             self.printer.emit_node(
@@ -116,6 +101,14 @@ class DiagramWriter:
                 label=rel.name,
                 type_=EdgeType.ASSOCIATION,
             )
+        # generate aggregations
+        for rel in diagram.get_relationships("aggregation"):
+            self.printer.emit_edge(
+                rel.from_object.fig_id,
+                rel.to_object.fig_id,
+                label=rel.name,
+                type_=EdgeType.AGGREGATION,
+            )
 
     def set_printer(self, file_name: str, basename: str) -> None:
         """Set printer."""
@@ -143,7 +136,7 @@ class DiagramWriter:
     def get_shape_color(self, obj: DiagramEntity) -> str:
         """Get shape color."""
         qualified_name = obj.node.qname()
-        if modutils.is_standard_module(qualified_name.split(".", maxsplit=1)[0]):
+        if modutils.is_stdlib_module(qualified_name.split(".", maxsplit=1)[0]):
             return "grey"
         if isinstance(obj.node, nodes.ClassDef):
             package = qualified_name.rsplit(".", maxsplit=2)[0]
